import requests


class UnsplashBase:

    base_url = 'https://api.unsplash.com/'
    fixed_profile = 'simonstaehli'

    def __init__(self, access_key: str) -> None:
        super().__init__()
        self.access_key = access_key
        self.session = requests.Session()
        self._check_status()
        
    def _check_status(self):
        """
        Checks if the API is reachable.

        Returns:
            API Status Code:    200 if everything is OK. 
        """
        response = self.session.get(self.base_url)
        assert response.status_code == 200, f'API not reachable Code: {response.status_code}'

        return response.status_code

    def get_current_rate_limit(self):
        """
        Returns the current rate Limit

        Returns:
            Remaining Requests.
        """
        response = requests.get(f'https://api.unsplash.com/users/{self.fixed_profile}', 
                                params=dict(client_id=self.access_key))
        response_headers = dict(response.headers)

        return response_headers.get('X-Ratelimit-Remaining')

    def get_headers(self):
        response = requests.get(f'https://api.unsplash.com/users/{self.fixed_profile}', 
                                params=dict(client_id=self.access_key))

        return response.headers


import unittest
from UnsplashAPI.collection import UnsplashCollections
from UnsplashAPI.photos import UnsplashPhotos, UnsplashSearch
from UnsplashAPI.topics_stats import UnsplashStats, UnsplashTopics
from UnsplashAPI.user import UnsplashUsers
import os

# Get Credentials for API
LOAD_FROM_ENV = True

if LOAD_FROM_ENV:
    access_keys = [os.environ['ACCESSKEY'], os.environ['ACCESSKEY2'], os.environ['ACCESSKEY3']]
else:
    with open('./tests/keys.txt', 'r') as txt_file:
        access_keys = [key.strip() for key in txt_file]

class TestUnsplashCollections(unittest.TestCase):
    
    def test_init(self):
        self.api = UnsplashCollections(access_key=access_keys[0])
        self.assertIsInstance(self.api, UnsplashCollections)

class TestUnsplashPhotos(unittest.TestCase):
    
    def test_init(self):
        self.api = UnsplashPhotos(access_key=access_keys[0])
        self.assertIsInstance(self.api, UnsplashPhotos)

class TestUnsplashSearch(unittest.TestCase):
    
    def test_init(self):
        self.api = UnsplashSearch(access_key=access_keys[0])
        self.assertIsInstance(self.api, UnsplashSearch)

class TestUnsplashStats(unittest.TestCase):

    def test_init(self):
        self.api = UnsplashStats(access_key=access_keys[0])
        self.assertIsInstance(self.api, UnsplashStats)

class TestUnsplashTopics(unittest.TestCase):
    
    def test_init(self):
        self.api = UnsplashTopics(access_key=access_keys[0])
        self.assertIsInstance(self.api, UnsplashTopics)

class TestUnplashUsers(unittest.TestCase):
    
    def test_init(self):
        self.api = UnsplashUsers(access_key=access_keys[0])
        self.assertIsInstance(self.api, UnsplashUsers)




import requests
from .base import UnsplashBase


class UnsplashPhotos(UnsplashBase):
    """
    This class implements API functionalities for the subcategory
    Photos.
    API-Doc: https://unsplash.com/documentation#list-photos
    """

    def __init__(self, access_key: str):
        """


        Args:
            access_key (str):           Access Key given from the API
        """
        super().__init__(access_key=access_key)
        self.access_key = access_key

    def list_photos_paginate(self, page_limit: int = 10, items_per_page: int = 10, **kwargs):
        """
        Get a mulitple pages from the Editorial feed.
        see here: https://unsplash.com/documentation#list-photos

        Args:
            page_limit (int, optional):     Defines Iteration Limit, when iterating over multiple pages. 
                                            Defaults to 10.
            items_per_page (int, optional): Defines how many items per page if multiple pages should be extracted
                                            Defaults to 10.
            **kwargs:
                    order_by: str = 'latest'

        Raises:
            Exception: Exception

        Yields:
            Dictionary with page contents: 
        """  
        endpoint = self.base_url + '/photos'
        for i in range(page_limit):
            response = self.session.get(endpoint,
                                        params=dict(client_id=self.access_key,
                                                    page=i,
                                                    per_page=items_per_page))
            if not response.status_code == 200:
                raise Exception(f'Not able to extract content. Code - {response.status_code}')

            yield response.json()

    def list_photos(self, items_per_page: int = 10, **kwargs):
        """
        Get a single page from the Editorial feed.
        see here: https://unsplash.com/documentation#list-photos

        Parameters
        ----------
        items_per_page:      Defines how many items per page if multiple pages should be extracted
        **kwargs:
                order_by: str = 'latest'

        Returns
        -------
        Dictionary of Items
        """
        endpoint = self.base_url + '/photos'
        response = self.session.get(endpoint,
                                    params=dict(client_id=self.access_key,
                                                page=1,
                                                per_page=items_per_page, **kwargs))
        if not response.status_code == 200:
            raise Exception(f'Not able to extract content. Code - {response.status_code}')

        return response.json()

    def get_photo_by_id(self, photo_id):
        """
        Retrieve a single photo.
        see here: https://unsplash.com/documentation#get-a-photo

        Parameters
        ----------
        photo_id:       ID of the photo
                        (see @ end of url if photo openend in browser)

        Returns
        -------
        Dictionary of Items
        """
        endpoint = self.base_url + f'/photos/{photo_id}'
        response = self.session.get(endpoint,
                                    params=dict(client_id=self.access_key))
        if not response.status_code == 200:
            raise Exception(f'Not able to extract content. Code - {response.status_code}')

        return response.json()

    def get_random_photo(self, **kwargs):
        """
        Returns a random photo.
        see here: https://unsplash.com/documentation#get-a-random-photo


        Parameters
        ----------
        kwargs:
            collections:	Public collection ID(‘s) to filter selection. If multiple, comma-separated
            topics:	        Public topic ID(‘s) to filter selection. If multiple, comma-separated
            username:	    Limit selection to a single user.
            query:	        Limit selection to photos matching a search term.
            orientation:	Filter by photo orientation. (Valid values: landscape, portrait, squarish)
            content_filter:	Limit results by content safety. Default: low. Valid values are low and high.
            count:	        The number of photos to return. (Default: 1; max: 30)

        Returns
        -------
        Dictionary of Items
        """
        endpoint = self.base_url + '/photos/random'
        response = self.session.get(endpoint,
                                    params=dict(client_id=self.access_key, **kwargs))
        if not response.status_code == 200:
            raise Exception(f'Not able to extract content. Code - {response.status_code}')

        return response.json()

    def get_photo_statistics(self, photo_id):
        """
        Retrieves statistics of a single photo.
        see here: https://unsplash.com/documentation#get-a-photos-statistics

        Parameters
        ----------
        photo_id:       ID of the photo
                        (see @ end of url if photo openend in browser)

        Returns
        -------
        Dictionary of Items
        """
        endpoint = self.base_url + f'/photos/{photo_id}/statistics'
        response = self.session.get(endpoint,
                                    params=dict(client_id=self.access_key))
        if not response.status_code == 200:
            raise Exception(f'Not able to extract content. Code - {response.status_code}')

        return response.json()

    def track_photo_download(self, photo_id):
        """
        Tracks donwload of a photo.

        see here: https://unsplash.com/documentation#track-a-photo-download

        Parameters
        ----------
        photo_id:       ID of the photo
                        (see @ end of url if photo openend in browser)

        Returns
        -------
        Dictionary of Items
        """
        endpoint = self.base_url + f'/photos/{photo_id}/download'
        response = self.session.get(endpoint,
                                    params=dict(client_id=self.access_key))
        if not response.status_code == 200:
            raise Exception(f'Not able to extract content. Code - {response.status_code}')

        return response.json()

    def update_photo(self, photo_id, **kwargs):
        """
        Updating a user photo. Requires Write Access for user profile (API-Settings)
        see here: https://unsplash.com/documentation#update-a-photo

        Parameters
        ----------
        photo_id:       ID of the photo
                        (see @ end of url if photo openend in browser)
        kwargs:
                id	                    The photo’s ID. Required.
                description	            The photo’s description (Optional).
                show_on_profile	        The photo’s visibility (Optional).
                tags	                The photo’s tags (Optional).
                location[latitude]	    The photo location’s latitude rounded to 6 decimals. (Optional)
                location[longitude]	    The photo location’s longitude rounded to 6 decimals. (Optional)
                location[name]	        The photo’s full location string (including city and country) (Optional)
                location[city]	        The photo location’s city (Optional)
                location[country]	    The photo location’s country (Optional)
                exif[make]	            Camera’s brand (Optional)
                exif[model]	            Camera’s model (Optional)
                exif[exposure_time]	    Camera’s exposure time (Optional)
                exif[aperture_value]	Camera’s aperture value (Optional)
                exif[focal_length]	    Camera’s focal length (Optional)
                exif[iso_speed_ratings]	Camera’s iso (Optional)

        Returns
        -------
        Dictionary of Items
        """
        endpoint = self.base_url + f'/photos/{photo_id}'
        response = self.session.put(endpoint,
                                    params=dict(client_id=self.access_key,
                                                **kwargs))

        return response.json()

    def like_photo(self, photo_id):
        """
        Likes a photo. Needs write access within API-settings.
        see here: https://unsplash.com/documentation#like-a-photo

        Parameters
        ----------
        photo_id:       ID of the photo
                        (see @ end of url if photo openend in browser)

        Returns
        -------
        Dictionary of Items
        """
        endpoint = self.base_url + f'/photos/{photo_id}/like'
        response = self.session.post(endpoint,
                                     params=dict(client_id=self.access_key))

        return response.json()

    def unlike_photo(self, photo_id):
        """
        Unlikes a photo. Needs write access in API settings.
        see here: https://unsplash.com/documentation#unlike-a-photo

        Parameters
        ----------
        photo_id:       ID of the photo
                        (see @ end of url if photo openend in browser)

        Returns
        -------
        Dictionary of Items
        """
        endpoint = self.base_url + f'/photos/{photo_id}/like'
        response = self.session.delete(endpoint,
                                       params=dict(client_id=self.access_key))

        return response.json()


class UnsplashSearch(UnsplashBase):
    """
    This class wraps the Search endpoint of the API.
    Docs: https://unsplash.com/documentation#search
    """

    def __init__(self, access_key: str) -> None:
        super().__init__(access_key=access_key)

    def search_photos(self, query: str, number_of_pages: int = 1, items_per_page: int = 10, **kwargs) -> iter:
        """
        Get a single page of photo results for a query.
        see here: https://unsplash.com/documentation#search

        Args:
            query (str):               Search Query
            number_of_pages (int):     Number of pages to retrieve.
                                       Default 1
            items_per_page (int):      Number of items per page
                                       Default: 10
            **kwargs:
                    per_page: Number of items per page. (Optional; default: 10)
                    order_by: How to sort the photos. (Optional; default: relevant). Valid values are latest and relevant.
                    collections	Collection ID(‘s) to narrow search. Optional. If multiple, comma-separated.
                    content_filter	Limit results by content safety. (Optional; default: low). Valid values are low and high.
                    color	Filter results by color. Optional. Valid values are: black_and_white, black, white, yellow, orange, red, purple, magenta, green, teal, and blue.
                    orientation	Filter by photo orientation. Optional. (Valid values: landscape, portrait, squarish)

        Returns:
            iter:                      Generator containing all elements.
        """
        endpoint = self.base_url + '/search/photos'
        for i in range(number_of_pages):
            response = self.session.get(endpoint,
                                        params=dict(client_id=self.access_key,
                                                    query=query,
                                                    page=i, 
                                                    per_page=items_per_page, 
                                                    **kwargs))
            if not response.status_code == 200:
                raise Exception(f'Not able to extract content. Code - {response.status_code}')

            yield response.json()

    def search_collections(self, query: str, number_of_pages: int = 1, items_per_page: int = 10) -> iter:
        """
        Get a single page of collection results for a query.
        see here:  https://unsplash.com/documentation#search-collections


        Args:
            query (str):                        Search query
            number_of_pages (int, optional):    Number of pages to retrieve.
                                                Defaults to 1.
            items_per_page (int, optional):     Number of items per page
                                                Defaults to 10.                        

        Returns:
            iter:               Generator containing all elements
        """
        endpoint = self.base_url + '/search/collections'
        for i in range(number_of_pages):
            response = self.session.get(endpoint,
                                        params=dict(client_id=self.access_key,
                                                    query=query,
                                                    page=i, 
                                                    per_page=items_per_page))
            if not response.status_code == 200:
                raise Exception(f'Not able to extract content. Code - {response.status_code}')

            yield response.json()


    def search_users(self, query: str, number_of_pages: int = 1, items_per_page: int = 10) -> iter:
        """
        Get a single page of user results for a query.
        see here:  https://unsplash.com/documentation#search-users


        Args:
            query (str):                        Search query
            number_of_pages (int, optional):    Number of pages to retrieve.
                                                Defaults to 1.
            items_per_page (int, optional):     Number of items per page
                                                Defaults to 10.                        

        Returns:
            iter:               Generator containing all elements
        """
        endpoint = self.base_url + '/search/users'
        for i in range(number_of_pages):
            response = self.session.get(endpoint,
                                        params=dict(client_id=self.access_key,
                                                    query=query,
                                                    page=i, 
                                                    per_page=items_per_page))
            if not response.status_code == 200:
                raise Exception(f'Not able to extract content. Code - {response.status_code}')

            yield response.json()
    













import unittest
from UnsplashAPI.base import UnsplashBase
import os

LOAD_FROM_ENV = True

if LOAD_FROM_ENV:
    access_keys = [os.environ['ACCESSKEY'], os.environ['ACCESSKEY2'], os.environ['ACCESSKEY3']]
else:
    with open('./tests/keys.txt', 'r') as txt_file:
        access_keys = [key.strip() for key in txt_file]

class TestUnsplashBase(unittest.TestCase):

    def test_init(self):
        base = UnsplashBase(access_key=access_keys[0])
        self.assertIsInstance(base, UnsplashBase)

    def test_check_api_con(self):
        base = UnsplashBase(access_key=access_keys[0])
        api_status = base._check_status()
        self.assertEqual(200, api_status)


import requests
from .base import UnsplashBase


class UnsplashCollections(UnsplashBase):
    """
    This class implements API functionalities for the subcategory
    Collections.
    API-Doc: https://unsplash.com/documentation#list-collections
    """

    def __init__(self, access_key: str):
        """


        Args:
            access_key (str):       Access Key of the API
        """ 
        super().__init__(access_key=access_key)
        self.access_key = access_key


    def list_collections_paginate(self, page_limit: int = 10, items_per_page: int = 10):
        """
        Get a mulitple pages from collections.
        see here: https://unsplash.com/documentation#list-collections

        Parameters
        ----------
        page_limit:             Defines Iteration Limit, when iterating over multiple pages.
        items_per_page:         Defines how many items per page if multiple pages should be extracted

        Returns
        -------
        Iterator
        """
        endpoint = self.base_url + '/collections'
        for i in range(page_limit):
            response = self.session.get(endpoint,
                                        params=dict(client_id=self.access_key,
                                                    page=i,
                                                    per_page=items_per_page))
            if not response.status_code == 200:
                raise Exception(f'Not able to extract content. Code - {response.status_code}')

            yield response.json()

    def list_collection(self, page: int = 1, items_per_page: int = 10):
        """
        Get a single page from the Editorial feed.
        see here: https://unsplash.com/documentation#list-collections

        Parameters
        ----------
        page:                Page to extract data from 
        items_per_page:      Defines how many items per page should be extracted

        Returns
        -------
        Dictionary of Items
        """
        endpoint = self.base_url + '/collections'
        response = self.session.get(endpoint,
                                    params=dict(client_id=self.access_key,
                                                page=page,
                                                per_page=items_per_page))
        if not response.status_code == 200:
            raise Exception(f'Not able to extract content. Code - {response.status_code}')

        return response.json()

    def get_collection_by_id(self, collection_id):
        """
        Retrieve a collection by its ID.
        see here: https://unsplash.com/documentation#get-a-collection

        Parameters
        ----------
        collection_id:   ID of the photo

        Returns
        -------
        Dictionary of Items
        """
        endpoint = self.base_url + f'/collections/{collection_id}'
        response = self.session.get(endpoint,
                                    params=dict(client_id=self.access_key))
        if not response.status_code == 200:
            raise Exception(f'Not able to extract content. Code - {response.status_code}')

        return response.json()

    def get_collection_photos(self, collection_id, page_limit: int = 10, per_page: int = 10,
                               **kwargs):
        """
        Returns pageable object. More infos: https://unsplash.com/documentation#get-a-collections-photos

        Args:
            collection_id (_type_):             ID of the collection
            page_limit (int, optional):         Paginating Limit 
                                                Defaults to 10.
            per_page (int, optional):           Number of Elements per page
                                                Defaults to 10.
            **kwargs:                           see here https://unsplash.com/documentation#get-a-collections-photos

        Raises:
            Exception: Response Status

        Yields:
            Dictionary with Elements
        """
        endpoint = self.base_url + f'/collections/{collection_id}/photos'

        for i in range(page_limit):
            response = self.session.get(endpoint,
                                        params=dict(client_id=self.access_key,
                                                    page=i,
                                                    per_page=per_page, 
                                                    **kwargs))
            if not response.status_code == 200:
                raise Exception(f'Not able to extract content. Code - {response.status_code}')

            yield response.json()

    def get_related_collections(self, collections_id):
        """
        Retrieve a list of collections related to this one.
        see here: https://unsplash.com/documentation#list-a-collections-related-collections


        Args:
            collections_id (_type_):       ID of the collection

        Raises:
            Exception: Exception

        Returns:
            _type_: _description_
        """
        endpoint = self.base_url + f'/collections/{collections_id}/related'

        response = self.session.get(endpoint,
                                    params=dict(client_id=self.access_key))

        if not response.status_code == 200:
            raise Exception(f'Not able to extract content. Code - {response.status_code}')

        return response.json()

    def create_new_collection(self, title: str, **kwargs):
        """
        Create a new collection. This requires the write_collections scope.
        see here: https://unsplash.com/documentation#create-a-new-collection

        Args:
            title (str):        Title of the collection 
            kwargs;             see here: https://unsplash.com/documentation#create-a-new-collection

        Raises:
            Exception: Exception

        Returns:
            response: Responds with the new collection:
        """

        endpoint = self.base_url + f'/collections'
        response = self.session.post(endpoint,
                                     params=dict(client_id=self.access_key, 
                                                 title=title, **kwargs))
        if not response.status_code == 200:
            raise Exception(f'Not able to extract content. Code - {response.status_code}')

        return response.json()

    def update_photo(self, photo_id, **kwargs):
        endpoint = self.base_url + f'/photos/{photo_id}'
        response = self.session.put(endpoint,
                                    params=dict(client_id=self.access_key,
                                                **kwargs))

        return response.json()

    def update_collection(self, collections_id, **kwargs):
        """
        Update an existing collection belonging to the logged-in user. This requires the write_collections scope.
        see here: https://unsplash.com/documentation#update-an-existing-collection

        Args:
            collections_id:         Collection ID
            **kwargs                https://unsplash.com/documentation#update-an-existing-collection

        Returns:
            response: Responds with the new collection:
        """

        endpoint = self.base_url + f'/collections/{collections_id}'
        response = self.session.put(endpoint,
                                     params=dict(client_id=self.access_key, 
                                                **kwargs))

        return response.json()

    def delete_collection(self, collections_id):
        """
        Delete a collection belonging to the logged-in user. This requires the write_collections scope.
        see here: https://unsplash.com/documentation#delete-a-collection

        Args:
            collections_id (_type_): _description_

        Returns:
            response: Responds with a 204 status and an empty body.
        """
        endpoint = self.base_url + f'/collections/{collections_id}'
        response = self.session.delete(endpoint,
                                       params=dict(client_id=self.access_key))

        return response.json()

    def add_photo_to_collection(self, collections_id, photo_id):
        """
        Add a photo to one of the logged-in user’s collections. Requires the write_collections scope.

        see here; https://unsplash.com/documentation#add-a-photo-to-a-collection

        Note: If the photo is already in the collection, this acion has no effect.

        Args:
            collections_id (_type_):        ID of the collection
            photo_id (_type_):              Id of the image to add to the collection
        """
        endpoint = self.base_url + f'/collections/{collections_id}/add'
        response = self.session.post(endpoint,
                                       params=dict(client_id=self.access_key, 
                                                   photo_id=photo_id))

        return response.json()

    def remove_photo_from_collection(self, collections_id, photo_id):
        """
        Remove a photo from one of the logged-in user’s collections. Requires the write_collections scope.
        
        see here: https://unsplash.com/documentation#remove-a-photo-from-a-collection

        Args:
            collections_id (_type_):        ID of the collection
            photo_id (_type_):              ID of the photo to remove from collection.
        """
        endpoint = self.base_url + f'/collections/{collections_id}/remove'
        response = self.session.delete(endpoint,
                                       params=dict(client_id=self.access_key, 
                                                   photo_id=photo_id))

        return response.json()

    








from .base import UnsplashBase
import requests


class UnsplashTopics(UnsplashBase):
    """
    Wrapper for API Methods Topics
    Doc: https://unsplash.com/documentation#topics
    """

    def __init__(self, access_key: str) -> None:
        """
        Args:
            access_key (str):       Access Key for the API
        """
        super().__init__(access_key)

    def get_topics(self, ids: str, number_of_pages: int = 1, items_per_page: int = 10, order_by: str = 'position') -> iter:
        """
        Get a single page from the list of all topics.
        see: https://unsplash.com/documentation#topics


         Args:
             ids (str):                         Limit to only matching topic ids or slugs. (Optional; Comma separated string) 
             number_of_pages (int, optional):   Number of total pages to retrieve. 
                                                Defaults to 1.
             items_per_page (int, optional):    Number of items per page. 
                                                Defaults to 10.
             order_by (str. optional):          How to sort the topics. (Optional; Valid values: featured, latest, oldest, position
                                                Defaults to position
         Raises:
             Exception: Exception

         Yields:
             Iterator[iter]:                    Iterator with list of all topics. 
        """
        
        endpoint = self.base_url + '/topics'
        for i in range(number_of_pages):
            response = self.session.get(endpoint,
                                        params=dict(client_id=self.access_key,
                                                    ids=ids,
                                                    page=i, 
                                                    per_page=items_per_page))
            if not response.status_code == 200:
                raise Exception(f'Not able to extract content. Code - {response.status_code}')

            yield response.json()

    def get_single_topic(self, ids: str) -> dict:
        """
        Get a single topic.
        see: https://unsplash.com/documentation#get-a-topic

         Args:
             ids (str):     Limit to only matching topic ids or slugs. (Optional; Comma separated string) 
         
         Exception: Exception

         Returns:
             Dict:          Dict with topic contents.
        """
        endpoint = self.base_url + '/topics'
        response = self.session.get(endpoint,
                                    params=dict(client_id=self.access_key,
                                                ids=ids))
        if not response.status_code == 200:
            raise Exception(f'Not able to extract content. Code - {response.status_code}')

        return response.json()

    def get_topic_photos(self, ids: str, number_of_pages: int = 1, items_per_page: int = 10, order_by: str = 'latest', **kwargs) -> iter:
        """
        Retrieve a topic’s photos.
        see: https://unsplash.com/documentation#get-a-topics-photos


         Args:
             ids (str):                         Limit to only matching topic ids or slugs. (Optional; Comma separated string) 
             number_of_pages (int, optional):   Number of total pages to retrieve. 
                                                Defaults to 1.
             items_per_page (int, optional):    Number of items per page. 
                                                Defaults to 10.
             order_by (str. optional):          How to sort the topics. (Optional; Valid values: featured, latest, oldest, position
                                                Defaults to latest
             **kwargs:
                        orientation	Filter by photo orientation. (Optional; Valid values: landscape, portrait, squarish)

         Raises:
             Exception: Exception

         Yields:
             Iterator[iter]:                    Iterator with list of all topics. 
        """
        
        endpoint = self.base_url + '/topics'
        for i in range(number_of_pages):
            response = self.session.get(endpoint,
                                        params=dict(client_id=self.access_key,
                                                    ids=ids,
                                                    page=i, 
                                                    per_page=items_per_page, **kwargs))
            if not response.status_code == 200:
                raise Exception(f'Not able to extract content. Code - {response.status_code}')

            yield response.json()
   

class UnsplashStats(UnsplashBase):
    """
    Wrapper for API endpoint stats. Docs: https://unsplash.com/documentation#stats
    """

    def __init__(self, access_key: str) -> None:
        """_summary_

        Args:
            access_key (str): _description_
        """
        super().__init__(access_key)

    def get_stats_total(self) -> dict:
        """
        Get a list of counts for all of Unsplash.
        see here: https://unsplash.com/documentation#totals

        Raises:
            Exception: _description_

        Returns:
            dict:   Dictionary with total stats.
        """
        endpoint = self.base_url + '/stats/total'
        response = self.session.get(endpoint,
                                    params=dict(client_id=self.access_key))
        if not response.status_code == 200:
            raise Exception(f'Not able to extract content. Code - {response.status_code}')

        return response.json()


    def get_stats_month(self) -> dict:
        """
        Get the overall Unsplash stats for the past 30 days.
        see here: https://unsplash.com/documentation#month

        Raises:
            Exception: Exception

        Returns:
            dict:   Dictionary with monthly stats. 
        """
        endpoint = self.base_url + '/stats/month'
        response = self.session.get(endpoint,
                                    params=dict(client_id=self.access_key))
        if not response.status_code == 200:
            raise Exception(f'Not able to extract content. Code - {response.status_code}')

        return response.json()


import requests
from .base import UnsplashBase


class UnsplashUsers(UnsplashBase):
    """
    Wraps the API methods of Users
    see here: https://unsplash.com/documentation#users
    """
    def __init__(self, access_key: str):
        """
        Args:
            access_key (str):        Access key of the API.
        """
        super().__init__(access_key=access_key)
        self.access_key = access_key

    def get_current_user(self) -> dict:
        """
        Retrieve public details on a given user.

        Raises:
            Exception: Excepion

        Returns:
            Dictionary:     Dict with profile contents.
        """
        endpoint = self.base_url + '/me'
        response = self.session.get(endpoint,
                                    params=dict(client_id=self.access_key))
        if not response.status_code == 200:
            raise Exception(f'Not able to extract content. Code - {response.status_code}')

        return response.json()

    def get_user_profile(self, username: str) -> dict:
        """
        Retrieve public details on a given user.

        Args:
            username (str):   Username 

        Raises:
            Exception: Exception

        Returns:
            Dict:  Dict containing profile elements
        """
        endpoint = self.base_url + f'/users/{username}'
        response = self.session.get(endpoint,
                                    params=dict(client_id=self.access_key))
        if not response.status_code == 200:
            raise Exception(f'Not able to extract content. Code - {response.status_code}')

        return response.json()

    def get_user_portfolio_link(self, username: str) -> dict:
        """
        Retrieve a single user’s portfolio link.
        see here: https://unsplash.com/documentation#get-a-users-portfolio-link

        Args:
            username (str):  Name of the user

        Raises:
            Exception: Exception

        Returns:
            dict:               dict containing portfolio link.
        """
        endpoint = self.base_url + f'/users/{username}/portfolio'
        response = self.session.get(endpoint,
                                    params=dict(client_id=self.access_key))
        if not response.status_code == 200:
            raise Exception(f'Not able to extract content. Code - {response.status_code}')

        return response.json()

    def list_user_photos(self, username: str) -> dict:
        """
        Get a list of photos uploaded by a user.
        see here: https://unsplash.com/documentation#list-a-users-photos

        Args:
            username (str):     Name of the user

        Raises:
            Exception: Exception

        Returns:
            dict:               dict containing all photos of a user.
        """
        endpoint = self.base_url + f'/users/{username}/photos'
        response = self.session.get(endpoint,
                                    params=dict(client_id=self.access_key))
        if not response.status_code == 200:
            raise Exception(f'Not able to extract content. Code - {response.status_code}')

        return response.json()

    def list_user_liked_photos(self, username: str) -> dict:
        """
        Get a list of photos liked by a user.
        see here: https://unsplash.com/documentation#list-a-users-liked-photos

        Args:
            username (str):     Name of the user

        Raises:
            Exception: Exception

        Returns:
            dict:               Liked photos of the user.
        """
        endpoint = self.base_url + f'/users/{username}/likes'
        response = self.session.get(endpoint,
                                    params=dict(client_id=self.access_key))
        if not response.status_code == 200:
            raise Exception(f'Not able to extract content. Code - {response.status_code}')

        return response.json()

    def list_user_liked_collections(self, username: str) -> dict:
        """
        Get a list of collections created by the user.
        see here; https://unsplash.com/documentation#list-a-users-collections

        Args:
            username (str):     Name of the user

        Raises:
            Exception: Exceptino

        Returns:
            dict:               Liked collections
        """

        endpoint = self.base_url + f'/users/{username}/collections'
        response = self.session.get(endpoint,
                                    params=dict(client_id=self.access_key))
        if not response.status_code == 200:
            raise Exception(f'Not able to extract content. Code - {response.status_code}')

        return response.json()

    def get_user_statistics(self, username: str) -> dict:
        """
        Retrieve the consolidated number of downloads, views and likes of all user’s photos, 
        as well as the historical breakdown and average of these stats in a specific timeframe (default is 30 days).
        see here: https://unsplash.com/documentation#get-a-users-statistics

        Args:
            username (str):         Name of user

        Raises:
            Exception: Exception

        Returns:
            dict:                   User stats
        """
        endpoint = self.base_url + f'/users/{username}/statistics'
        response = self.session.get(endpoint,
                                    params=dict(client_id=self.access_key))
        if not response.status_code == 200:
            raise Exception(f'Not able to extract content. Code - {response.status_code}')

        return response.json()








from setuptools import setup, find_packages

VERSION = '0.0.3'
DESCRIPTION = 'Inofficial API Wrapper for Unsplash API'
LONG_DESCRIPTION = 'A package which allow python users to easily interact with the UnplashAPI'

# Setting up
setup(
    name="unsplashapi",
    version=VERSION,
    author="SimonStaehli",
    author_email="<simon.staehli@students.fhnw.ch>",
    description=DESCRIPTION,
    long_description_content_type="text/markdown",
    long_description=LONG_DESCRIPTION,
    packages=find_packages(),
    install_requires=['requests'],
    keywords=['python', 'api', 'unsplash api', 'unsplash'],
    python_requires='>=3',
    classifiers=[
        "Development Status :: 1 - Planning",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Operating System :: Unix",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
    ]
)


import unittest
from unittest import result
from urllib import response
from xmlrpc.client import ResponseError
import os
import time
#import sys
#sys.path.insert(0, '..')
from UnsplashAPI.api import UnsplashAPI


LOAD_FROM_ENV = True

if LOAD_FROM_ENV:
    access_keys = [os.environ['ACCESSKEY'], os.environ['ACCESSKEY2'], os.environ['ACCESSKEY3']]
else:
    with open('keys.txt', 'r') as txt_file:
        access_keys = [key.strip() for key in txt_file]


class TestAPICollections(unittest.TestCase):
    """
    This class implements test for collection endpoint.
    This class only tests responses from endpoints with read access.
    """
    # Params
    page_limit = 2
    items_per_page = 5
    # Dict keys to check
    check_keys = ['id', 'title', 'description', 'published_at']
    # Check collection IDS
    collection_id = 2001768
    collection_title = 'Life in the Deep'

    access_key_idx = 0

    # Init API
    api = UnsplashAPI(access_key=access_keys[access_key_idx])

    # General Test
    def test_init(self):
        self.assertIsInstance(self.api, UnsplashAPI)

    # Collections
    def test_list_collections_paginate(self):
        try:
            response = self.api.list_collections_paginate(page_limit=self.page_limit, 
                                                        items_per_page=self.items_per_page)            
            results = [element for element in response]
        except Exception:
            # Workaround for Rate Limits
            rate_limit = int(self.api.get_current_rate_limit())
            if rate_limit == 0:
                print('No Remaining Rate Limits.', rate_limit)
                while rate_limit == 0:
                    self.access_key_idx += 1
                    if self.access_key_idx == len(access_keys):
                        self.access_key_idx = 0
                    self.api.access_key = access_keys[self.access_key_idx]
                    rate_limit = int(self.api.get_current_rate_limit())

            response = self.api.list_collections_paginate(page_limit=self.page_limit, 
                                                        items_per_page=self.items_per_page)            
            # Perform request again
            results = [element for element in response]

        self.assertEqual(self.page_limit, len(results))
        self.assertEqual(self.items_per_page, len(results[0]))

    def test_list_collection(self):
        try:    
            response = self.api.list_collection(page=1, items_per_page=self.items_per_page)
        except Exception:
            # Workaround for Rate Limits
            rate_limit = int(self.api.get_current_rate_limit())
            if rate_limit == 0:
                print('No Remaining Rate Limits.', rate_limit)
                while rate_limit == 0:
                    self.access_key_idx += 1
                    if self.access_key_idx == len(access_keys):
                        self.access_key_idx = 0
                    self.api.access_key = access_keys[self.access_key_idx]
                    rate_limit = int(self.api.get_current_rate_limit())
            # Perform request again
            response = self.api.list_collection(page=1, items_per_page=self.items_per_page)

        self.assertEqual(self.items_per_page, len(response))
        element = response[0]
        self.assertIsInstance(element, dict)
        for ckey in self.check_keys:
            self.assertIn(ckey, element.keys())
    
    def test_get_collection_by_id(self):
        try:
            response = self.api.get_collection_by_id(collection_id=self.collection_id)
        except Exception:
            # Workaround for Rate Limits
            rate_limit = int(self.api.get_current_rate_limit())
            if rate_limit == 0:
                print('No Remaining Rate Limits.', rate_limit)
                while rate_limit == 0:
                    self.access_key_idx += 1
                    if self.access_key_idx == len(access_keys):
                        self.access_key_idx = 0
                    self.api.access_key = access_keys[self.access_key_idx]
                    rate_limit = int(self.api.get_current_rate_limit())
            # Perform request again
            response = self.api.get_collection_by_id(collection_id=self.collection_id)

        self.assertIsInstance(response, dict)
        self.assertEqual(response['id'], str(self.collection_id))
        self.assertEqual(response['title'], self.collection_title)
    
    def test_get_collection_photos(self):
        try:
            response = self.api.get_collection_photos(collection_id=self.collection_id,
                                                    page_limit=self.page_limit, 
                                                    per_page=self.items_per_page)
            results = [element for element in response]
        except Exception:
            # Workaround for Rate Limits
            rate_limit = int(self.api.get_current_rate_limit())
            if rate_limit == 0:
                print('No Remaining Rate Limits.', rate_limit)
                while rate_limit == 0:
                    self.access_key_idx += 1
                    if self.access_key_idx == len(access_keys):
                        self.access_key_idx = 0
                    self.api.access_key = access_keys[self.access_key_idx]
                    rate_limit = int(self.api.get_current_rate_limit())
            print('Current Rate Limit', self.api.get_current_rate_limit())
            # Perform request again
            results = [element for element in response]

        self.assertEqual(self.page_limit, len(results))
        self.assertEqual(self.items_per_page, len(results[0]))
        self.assertIsInstance(results[0][0], dict)
    
    def test_get_related_collections(self):
        try:
            response = self.api.get_related_collections(collections_id=self.collection_id)
        except Exception:
            # Workaround for Rate Limits
            rate_limit = int(self.api.get_current_rate_limit())
            if rate_limit == 0:
                print('No Remaining Rate Limits.', rate_limit)
                while rate_limit == 0:
                    self.access_key_idx += 1
                    if self.access_key_idx == len(access_keys):
                        self.access_key_idx = 0
                    self.api.access_key = access_keys[self.access_key_idx]
                    rate_limit = int(self.api.get_current_rate_limit())
            # Perform request again
            response = self.api.get_related_collections(collections_id=self.collection_id)

        self.assertIsInstance(response, list)

        # Check keys in one response
        element = response[0]
        for ckey in self.check_keys:
            self.assertIn(ckey, element.keys())

class TestAPIPhotos(unittest.TestCase):
    """
    This class implements test for photos endpoint.
    This class only tests responses from endpoints with read access.
    """
    # Params
    page_limit = 2
    items_per_page = 5
    # Dict keys to check
    check_keys = ['id', 'description', 'urls', 'likes']
    # Check collection IDS
    photo_id = 'tCyI0KY9jTs'


    access_key_idx = 0
    # Init API
    api = UnsplashAPI(access_key=access_keys[access_key_idx])

    # General Test
    def test_init(self):
        self.assertIsInstance(self.api, UnsplashAPI)

    def test_list_photos_paginate(self):
        try:
            response = self.api.list_photos_paginate(page_limit=self.page_limit, 
                                                    items_per_page=self.items_per_page)            
            results = [element for element in response]
        except Exception:
            # Workaround for Rate Limits
            rate_limit = int(self.api.get_current_rate_limit())
            if rate_limit == 0:
                print('No Remaining Rate Limits.', rate_limit)
                while rate_limit == 0:
                    self.access_key_idx += 1
                    if self.access_key_idx == len(access_keys):
                        self.access_key_idx = 0
                    self.api.access_key = access_keys[self.access_key_idx]
                    rate_limit = int(self.api.get_current_rate_limit())

            response = self.api.list_photos_paginate(page_limit=self.page_limit, 
                                                        items_per_page=self.items_per_page)            
            # Perform request again
            results = [element for element in response]

        self.assertEqual(self.page_limit, len(results))
        self.assertEqual(self.items_per_page, len(results[0]))
    
    def test_list_photos(self):
        try:
            response = self.api.list_photos(items_per_page=self.items_per_page)            
        except Exception:
            # Workaround for Rate Limits
            rate_limit = int(self.api.get_current_rate_limit())
            if rate_limit == 0:
                print('No Remaining Rate Limits.', rate_limit)
                while rate_limit == 0:
                    self.access_key_idx += 1
                    if self.access_key_idx == len(access_keys):
                        self.access_key_idx = 0
                    self.api.access_key = access_keys[self.access_key_idx]
                    rate_limit = int(self.api.get_current_rate_limit())

            response = self.api.list_photos(page_limit=self.page_limit, 
                                                        items_per_page=self.items_per_page)            
        self.assertEqual(self.items_per_page, len(response))
        for key in self.check_keys:
            self.assertIn(key, response[0].keys())
        
    
    def test_get_photo_by_id(self):
        try:
            response = self.api.get_photo_by_id(photo_id=self.photo_id)            
        except Exception:
            # Workaround for Rate Limits
            rate_limit = int(self.api.get_current_rate_limit())
            if rate_limit == 0:
                print('No Remaining Rate Limits.', rate_limit)
                while rate_limit == 0:
                    self.access_key_idx += 1
                    if self.access_key_idx == len(access_keys):
                        self.access_key_idx = 0
                    self.api.access_key = access_keys[self.access_key_idx]
                    rate_limit = int(self.api.get_current_rate_limit())

            response = self.api.get_photo_by_id(photo_id=self.photo_id)            
        
        self.assertIsInstance(response, dict)
        # Add more tests here
    
    def test_get_random_photo(self):
        try:
            response = self.api.get_random_photo()            
        except Exception:
            # Workaround for Rate Limits
            rate_limit = int(self.api.get_current_rate_limit())
            if rate_limit == 0:
                print('No Remaining Rate Limits.', rate_limit)
                while rate_limit == 0:
                    self.access_key_idx += 1
                    if self.access_key_idx == len(access_keys):
                        self.access_key_idx = 0
                    self.api.access_key = access_keys[self.access_key_idx]
                    rate_limit = int(self.api.get_current_rate_limit())

            response = self.api.get_random_photo()            
        
        self.assertIsInstance(response, dict)
        # Add more tests here

    def test_get_photo_statistics(self):
        try:
            response = self.api.get_photo_statistics(photo_id=self.photo_id)            
        except Exception:
            # Workaround for Rate Limits
            rate_limit = int(self.api.get_current_rate_limit())
            if rate_limit == 0:
                print('No Remaining Rate Limits.', rate_limit)
                while rate_limit == 0:
                    self.access_key_idx += 1
                    if self.access_key_idx == len(access_keys):
                        self.access_key_idx = 0
                    self.api.access_key = access_keys[self.access_key_idx]
                    rate_limit = int(self.api.get_current_rate_limit())

            response = self.api.get_photo_statistics(photo_id=self.photo_id)            
        
        self.assertIsInstance(response, dict)
        # Add more tests here

    def test_track_photo_download(self):
        try:
            response = self.api.track_photo_download(photo_id=self.photo_id)            
        except Exception:
            # Workaround for Rate Limits
            rate_limit = int(self.api.get_current_rate_limit())
            if rate_limit == 0:
                print('No Remaining Rate Limits.', rate_limit)
                while rate_limit == 0:
                    self.access_key_idx += 1
                    if self.access_key_idx == len(access_keys):
                        self.access_key_idx = 0
                    self.api.access_key = access_keys[self.access_key_idx]
                    rate_limit = int(self.api.get_current_rate_limit())

            response = self.api.track_photo_download(photo_id=self.photo_id)            
        
        self.assertIsInstance(response, dict)
        # Add more tests here

    

    

    
    


    



from .collection import UnsplashCollections
from .photos import UnsplashPhotos, UnsplashSearch
from .user import UnsplashUsers
from .topics_stats import UnsplashStats, UnsplashTopics


class UnsplashAPI(UnsplashCollections, UnsplashPhotos,
                  UnsplashUsers, UnsplashSearch, UnsplashTopics, UnsplashStats):
    """
    This class combines all subclasses in one interface.
    """

    def __init__(self, access_key):
        """
        Inofficial Wrapper class for the Unsplash API. 

        All functions are named like the official endpoints, which can be found here: https://unsplash.com/documentation

        To get an access key:
            - Create Developer Account on Unsplash.com
            - Add new Application
            - Adjust "permissions" for your application
            - copy the "access key" in "Keys"

        Args:
            access_key (str):   Access key for application from offifical API.

        Usage:
            api = UnsplashAPI(access_key='<your key>')
            api.get_current_rate_limit()
            >>> '49'
        """
        UnsplashCollections.__init__(self, access_key=access_key)
        UnsplashPhotos.__init__(self, access_key=access_key)
        UnsplashUsers.__init__(self, access_key=access_key)
        UnsplashSearch.__init__(self, access_key=access_key)
        UnsplashTopics.__init__(self, access_key=access_key)
        UnsplashStats.__init__(self, access_key=access_key)
        self.access_key = access_key
